import {Component, EventEmitter, Output} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {SelectionModel} from "@angular/cdk/collections";
import {HttpClient} from "@angular/common/http";
import {ToastrService} from "ngx-toastr";
import {ActivatedRoute, Router} from "@angular/router";

@Component({
  selector: 'app-deploy-image',
  templateUrl: './deploy-image.component.html',
  styleUrl: './deploy-image.component.css'
})
export class DeployImageComponent {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  @Output() dataChange = new EventEmitter<any>();

  errorMessage = '';
  clientId: string | null = null;
  partitions: any[] = [];
  images: any[] = [];
  clientName: string = '';
  selectedImage: string | null = null;
  selectedOption: string | null = 'deploy-image';
  selectedMethod: string | null = null;
  selectedPartition: any = null;
  mcastIp: string = '';
  mcastPort: string = '';
  mcastMode: string = '';
  mcastSpeed: Number = 0;
  mcastMaxClients: Number = 0;
  mcastMaxTime: Number = 0;
  p2pMode: string = '';
  p2pTime: Number = 0;
  name: string = '';

  protected p2pModeOptions = [
    { name: 'Leecher', value: 'p2p-mode-leecher' },
    { name: 'Peer', value: 'p2p-mode-peer' },
    { name: 'Seeder', value: 'p2p-mode-seeder' },
  ];
  protected multicastModeOptions = [
    {"name": 'Half duplex', "value": "half-duplex"},
    {"name": 'Full duplex', "value": "full-duplex"},
  ];

  allMethods = [
    'multicast',
    'multicast-direct',
    'unicast',
    'unicast-direct',
    'torrent'
  ];

  updateCacheMethods = [
    'multicast',
    'unicast',
    'torrent'
  ];

  dataSource = new MatTableDataSource<any>();
  columns = [
    {
      columnDef: 'diskNumber',
      header: 'Disco',
      cell: (partition: any) => `${partition.diskNumber}`
    },
    {
      columnDef: 'partitionNumber',
      header: 'Particion',
      cell: (partition: any) => `${partition.partitionNumber}`
    },
    {
      columnDef: 'size',
      header: 'Tamaño',
      cell: (partition: any) => `${partition.size} MB`
    },
    {
      columnDef: 'filesystem',
      header: 'Sistema de ficheros',
      cell: (partition: any) => `${partition.filesystem}`
    },
    {
      columnDef: 'operativeSystem',
      header: 'SO',
      cell: (partition: any) => `${partition.operativeSystem?.name}`
    }
  ];

  displayedColumns = ['select', ...this.columns.map(column => column.columnDef)];
  selection = new SelectionModel(true, []);

  constructor(
    private http: HttpClient,
    private toastService: ToastrService,
    private route: ActivatedRoute,
    private router: Router,
  ) {}

  ngOnInit() {
    this.clientId = this.route.snapshot.paramMap.get('id');
    this.selectedOption = 'deploy-image';
    this.loadPartitions();
    this.loadImages();
  }

  get deployMethods() {
    return this.selectedOption === 'update-cache' ? this.updateCacheMethods : this.allMethods;
  }

  isMethod(method: string): boolean {
    return this.selectedMethod === method;
  }

  loadPartitions() {
    const url = `${this.baseUrl}/clients/${this.clientId}`;
    this.http.get(url).subscribe(
      (response: any) => {
        if (response.partitions) {
          this.clientName = response.name;
          this.dataSource.data = response.partitions.filter((partition: any) => {
            return partition.partitionNumber !== 0;
          });
          this.p2pMode = response.organizationalUnit?.networkSettings?.p2pMode;
          this.p2pTime = response.organizationalUnit?.networkSettings?.p2pTime;
          this.mcastSpeed = response.organizationalUnit?.networkSettings?.mcastSpeed;
          this.mcastMode = response.organizationalUnit?.networkSettings?.mcastMode;
          this.mcastPort = response.organizationalUnit?.networkSettings?.mcastPort;
          this.mcastIp = response.organizationalUnit?.networkSettings?.mcastIp;
        }
      },
      (error) => {
        console.error('Error al cargar los datos del cliente:', error);
      }
    );
  }

  loadImages() {
    const url = `${this.baseUrl}/images?status=success&page=1&itemsPerPage=1000`;
    this.http.get(url).subscribe(
      (response: any) => {
        this.images = response['hydra:member'];
      },
      (error) => {
        console.error('Error al cargar las imágenes:', error);
      }
    );
  }

  save(): void {
    if (!this.selectedImage) {
      this.toastService.error('Debe seleccionar una imagen');
      return;
    }

    if (!this.selectedMethod) {
      this.toastService.error('Debe seleccionar un método');
      return;
    }

    if (!this.selectedPartition) {
      this.toastService.error('Debe seleccionar una partición');
      return;
    }

    const payload = {
      client: `/clients/${this.clientId}`,
      method: this.selectedMethod,
      partition: this.selectedPartition['@id'],
      p2pMode: this.p2pMode,
      p2pTime: this.p2pTime,
      mcastIp: this.mcastIp,
      mcastPort: this.mcastPort,
      mcastMode: this.mcastMode,
      mcastSpeed: this.mcastSpeed,
    };

    this.http.post(`${this.baseUrl}${this.selectedImage}/deploy-image`, payload)
      .subscribe({
          next: (response) => {
            this.toastService.success('Imagen creada exitosamente');
            this.router.navigate(['/commmands-logs'])
          },
          error: (error) => {
            console.error('Error:', error);
            this.toastService.error(error.error['hydra:description']);
          }
        }
      );
  }
}
