import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { MatSnackBar } from '@angular/material/snack-bar';
import { ToastrService } from 'ngx-toastr';
import { DataService } from '../../../services/data.service';
import * as Papa from 'papaparse';

@Component({
  selector: 'app-create-client',
  templateUrl: './create-client.component.html',
  styleUrls: ['./create-client.component.css']
})
export class CreateClientComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  clientForm!: FormGroup;
  parentUnits: any[] = [];
  hardwareProfiles: any[] = [];
  ogLives: any[] = [];
  templates: any[] = [];
  uploadedClients: any[] = [];
  loading: boolean = false;
  displayedColumns: string[] = ['name', 'ip'];
  isSingleClientForm: boolean = false;
  showTextarea: boolean = true;
  protected netifaceTypes = [
    { name: 'Eth0', value: 'eth0' },
    { name: 'Eth1', value: 'eth1' },
    { name: 'Eth2', value: 'eth2' }
  ];
  protected netDriverTypes = [
    { name: 'Generic', value: 'generic' }
  ];

  constructor(
    private fb: FormBuilder,
    private dialogRef: MatDialogRef<CreateClientComponent>,
    private http: HttpClient,
    private snackBar: MatSnackBar,
    private toastService: ToastrService,
    private dataService: DataService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {}

  ngOnInit(): void {
    this.initForm();
    this.loadParentUnits();
    this.loadHardwareProfiles();
    this.loadOgLives();
    this.loadPxeTemplates();
  }

  initForm(): void {
    this.clientForm = this.fb.group({
      organizationalUnit: [
        this.data.organizationalUnit ? this.data.organizationalUnit['@id'] : null,
        Validators.required
      ],
      name: ['', Validators.required],
      serialNumber: [''],
      netiface: null,
      netDriver: null,
      mac: ['', Validators.required],
      ip: ['', Validators.required],
      template: [null],
      hardwareProfile: [
        this.data.organizationalUnit?.networkSettings?.hardwareProfile?.['@id'] || null
      ],
      ogLive: [null]
    });
  }

  loadParentUnits(): void {
    this.loading = true;
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
        this.loading = false;
      },
      error => {
        console.error('Error fetching parent units:', error);
        this.loading = false;
      }
    );
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => {
        this.hardwareProfiles = data;
        this.loading = false;
      },
      error => {
        console.error('Error fetching hardware profiles:', error);
        this.loading = false;
      }
    );
  }

  loadOgLives(): void {
    const url = `${this.baseUrl}/og-lives?page=1&itemsPerPage=30`;

    this.http.get<any>(url).subscribe(
      response => {
        this.ogLives = response['hydra:member'];
      },
      error => {
        console.error('Error fetching ogLives:', error);
      }
    );
  }

  loadPxeTemplates(): void {
    const url = `${this.baseUrl}/pxe-templates?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.templates = response['hydra:member'];
      },
      error => {
        console.error('Error fetching PXE templates:', error);
      }
    );
  }

  onFileUpload(event: any): void {
    const file = event.target.files[0];
    if (file) {
      const reader = new FileReader();
  
      reader.onload = (e: any) => {
        const textData = e.target.result;
        const regex = /host\s+(\S+)\s+\{\s+hardware\s+ethernet\s+([\da-fA-F:]+);\s+fixed-address\s+([\d.]+);\s+\}/g;
        let match;
        const clients = [];
  
        while ((match = regex.exec(textData)) !== null) {
          clients.push({
            name: match[1],
            mac: match[2],
            ip: match[3]
          });
        }
  
        if (clients.length > 0) {
          this.uploadedClients = clients;
          this.toastService.success('Archivo cargado correctamente, los datos están listos para enviarse.', 'Éxito');
          this.showTextarea = false;
        } else {
          this.toastService.error('No se encontraron datos válidos', 'Error');
          this.showTextarea = true;
        }
      };
  
      reader.readAsText(file);
    }
  }
  
  onTextarea(text: string): void {
    const regex = /host\s+(\S+)\s+\{\s+hardware\s+ethernet\s+([\da-fA-F:]+);\s+fixed-address\s+([\d.]+);\s+\}/g;
    let match;
    const clients = [];
  
    while ((match = regex.exec(text)) !== null) {
      clients.push({
        name: match[1],
        mac: match[2],
        ip: match[3]
      });
    }
  
    if (clients.length > 0) {
      this.uploadedClients = clients;
      this.toastService.success('Datos cargados correctamente, los datos están listos para enviarse.', 'Éxito');
      this.showTextarea = false;
    } else {
      this.toastService.error('No se encontraron datos válidos', 'Error');
      this.showTextarea = true;
    }
  }
  
  onSubmit(): void {
    if (this.isSingleClientForm) {
      if (this.clientForm.valid) {
        const formData = this.clientForm.value;
        console.log('Form data:', formData);
        this.http.post(`${this.baseUrl}/clients`, formData).subscribe(
          response => {
            this.toastService.success('Cliente creado exitosamente', 'Éxito');
            this.dialogRef.close(response);
          },
          error => {
            console.error('Error durante POST:', error);
            this.toastService.error('Error al crear el cliente', 'Error');
          }
        );
      }
    } else {
      if (this.uploadedClients.length > 0) {
        this.uploadedClients.forEach(client => {
          const formData = {
            organizationalUnit: this.clientForm.value.organizationalUnit || null,
            name: client.name || null,
            mac: client.mac || null,
            ip: client.ip || null,
            template: this.clientForm.value.template || null,
            hardwareProfile: this.clientForm.value.hardwareProfile || null,
            ogLive: this.clientForm.value.ogLive || null,
            serialNumber: null,
            netiface: null,
            netDriver: null
          };
  
          this.http.post(`${this.baseUrl}/clients`, formData).subscribe(
            response => {
              this.toastService.success(`Cliente ${client.name} creado exitosamente`, 'Éxito');
            },
            error => {
              console.error(`Error al crear el cliente ${client.name}:`, error);
              this.toastService.error(`Error al crear el cliente ${client.name}`, 'Error');
            }
          );
        });
  
        this.uploadedClients = [];
        this.dialogRef.close();
      } else {
        this.toastService.error('No hay clientes cargados para añadir', 'Error');
      }
    }
  }
  
  toggleClientForm(): void {
    this.isSingleClientForm = !this.isSingleClientForm;
  }
  
  onNoClick(): void {
    this.dialogRef.close();
  }
}
