import {Component, Inject} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {HttpClient} from "@angular/common/http";
import {ToastrService} from "ngx-toastr";
import {DataService} from "../../images/data.service";
import {ActivatedRoute} from "@angular/router";
import {MatTableDataSource} from "@angular/material/table";
import {DatePipe} from "@angular/common";
import {ServerInfoDialogComponent} from "../../ogdhcp/og-dhcp-subnets/server-info-dialog/server-info-dialog.component";
import {CreateImageComponent} from "../../images/create-image/create-image.component";
import {DeleteModalComponent} from "../../../shared/delete_modal/delete-modal/delete-modal.component";
import {Observable} from "rxjs";
import {MatDialog} from "@angular/material/dialog";

@Component({
  selector: 'app-main-repository-view',
  templateUrl: './main-repository-view.component.html',
  styleUrl: './main-repository-view.component.css'
})
export class MainRepositoryViewComponent {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  repositoryForm: FormGroup<any>;
  repositoryId: string | null = null;
  dataSource = new MatTableDataSource<any>();
  loading: boolean = true;
  diskUsage: any = {};
  servicesStatus: any = {};
  diskUsageChartData: any[] = [];
  alertMessage: string | null = null;
  length: number = 0;
  itemsPerPage: number = 10;
  page: number = 0;
  view: [number, number] = [800, 500];
  gradient: boolean = true;
  showLegend: boolean = true;
  showLabels: boolean = true;
  isDoughnut: boolean = true;
  repositoryData: any = {};
  colorScheme: any = {
    domain: ['#FF6384', '#3f51b5']
  };

  filters: { [key: string]: string } = {};
  datePipe: DatePipe = new DatePipe('es-ES');
  columns = [
    {
      columnDef: 'id',
      header: 'Id',
      cell: (image: any) => `${image.id}`
    },
    {
      columnDef: 'name',
      header: 'Nombre de imagen',
      cell: (image: any) => `${image.name}`
    },
    {
      columnDef: 'imageRepository',
      header: 'Repositorio',
      cell: (image: any) => `${image.imageRepository?.name}`
    },
    {
      columnDef: 'remotePc',
      header: 'Remote Pc',
      cell: (image: any) => `${image.remotePc}`
    },
    {
      columnDef: 'created',
      header: 'Creado en repositorio',
      cell: (image: any) => `${image.created}`
    },
    {
      columnDef: 'imageFullsum',
      header: 'Fullsum',
      cell: (image: any) => `${image.imageFullsum}`
    },
    {
      columnDef: 'createdAt',
      header: 'Fecha de creación',
      cell: (image: any) => `${this.datePipe.transform(image.createdAt, 'dd/MM/yyyy hh:mm:ss')}`
    }
  ];
  displayedColumns = [...this.columns.map(column => column.columnDef), 'actions'];
  private apiUrl = `${this.baseUrl}/images`;

  constructor(
    private fb: FormBuilder,
    private http: HttpClient,
    private toastService: ToastrService,
    private dataService: DataService,
    private route: ActivatedRoute,
    public dialog: MatDialog,
  ) {
    this.repositoryForm = this.fb.group({
      name: ['', Validators.required],
      ip: [''],
      comments: [''],
    });
  }

  ngOnInit() {
    this.repositoryId = this.route.snapshot.paramMap.get('id');
    this.loading = true
    this.load()
    this.loadStatus()
  }

  load(): void {
    const url = `${this.baseUrl}/image-repositories/${this.repositoryId}`;
    this.http.get(url).subscribe(
      (response: any) => {
        this.repositoryData = response;
        this.repositoryForm = this.fb.group({
          name: [response.name, Validators.required],
          ip: [response.ip],
          comments: [response.comments],
        });
        this.loading = false;
        // Llamar searchImages() solo cuando la data de repository esté cargada
        this.searchImages();
      },
      (error) => {
        console.error('Error al cargar los datos del cliente:', error);
        this.loading = false;
      }
    );
  }

  save(): void {
    const payload = {
      name: this.repositoryForm.value.name,
      ip: this.repositoryForm.value.ip,
      comments: this.repositoryForm.value.comments,
    };

    if (this.repositoryId) {
      this.http.put(`${this.baseUrl}/image-repositories/${this.repositoryId}`, payload).subscribe(
        (response) => {
          this.toastService.success('Imagen editada correctamente');
        },
        (error) => {
          this.toastService.error(error['error']['hydra:description']);
          console.error('Error al editar la imagen', error);
        }
      );
    } else {
      this.http.post(`${this.baseUrl}/image-repositories`, payload).subscribe(
        (response) => {
          this.toastService.success('Imagen añadida correctamente');
        },
        (error) => {
          this.toastService.error(error['error']['hydra:description']);
          console.error('Error al añadir la imagen', error);
        }
      );
    }
  }

  loadStatus(): void {
    this.http.get<any>(`${this.baseUrl}/image-repositories/server/${this.repositoryId}/status`).subscribe(data => {
      const diskData = data.output.disk;
      const servicesData = data.output.services;

      this.diskUsage = {
        total: diskData.total,
        used: diskData.used,
        available: diskData.available,
        percentage: diskData.used_percentage
      };

      this.diskUsageChartData = [
        {
          name: 'Usado',
          value: parseFloat(diskData.used)
        },
        {
          name: 'Disponible',
          value: parseFloat(diskData.available)
        }
      ];

      this.servicesStatus = servicesData;

    }, error => {
      console.error('Error fetching status', error);
    });
  }

  getServices(): { name: string, status: string }[] {
    return Object.keys(this.servicesStatus).map(key => ({
      name: key,
      status: this.servicesStatus[key]
    }));
  }

  searchImages(): void {
    this.loading = true;
    this.http.get<any>(`${this.apiUrl}?repository.id=${this.repositoryData.id}&page=${this.page +1 }&itemsPerPage=${this.itemsPerPage}`, { params: this.filters }).subscribe(
      data => {
        this.dataSource.data = data['hydra:member'];
        this.length = data['hydra:totalItems'];
        this.loading = false;
      },
      error => {
        console.error('Error fetching images', error);
        this.loading = false;
      }
    );
  }

  editImage(event: MouseEvent, image: any): void {
    event.stopPropagation();
    this.dialog.open(CreateImageComponent, {
      width: '800px',
      data: image['@id']
    }).afterClosed().subscribe(() => this.searchImages());
  }

  deleteImage(image: any): void {
    this.dialog.open(DeleteModalComponent, {
      width: '300px',
      data: { name: image.name },
    }).afterClosed().subscribe((result) => {
      if (result) {
        this.http.delete(`${this.apiUrl}/server/${image.uuid}/delete`).subscribe({
          next: () => {
            this.toastService.success('Imagen eliminada con éxito');
            this.searchImages();
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
            console.error('Error al eliminar la imagen:', error);
          }
        });
      }
    });
  }

  loadImageAlert(image: any): Observable<any> {
    return this.http.get<any>(`${this.apiUrl}/server/${image.uuid}/get`, {});
  }

  showImageInfo(event: MouseEvent, image:any) {
    event.stopPropagation();
    this.loadImageAlert(image).subscribe(
      response => {
        this.alertMessage = response.output;

        this.dialog.open(ServerInfoDialogComponent, {
          width: '600px',
          data: {
            message: this.alertMessage
          }
        });
      },
      error => {
        this.toastService.error(error.error['hydra:description']);
      }
    );
  }

  onPageChange(event: any): void {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
    this.length = event.length;
    this.searchImages();
  }

  loadAlert(): Observable<any> {
    return this.http.get<any>(`${this.baseUrl}/image-repositories/server/get-collection`);
  }

  syncRepository() {
    this.http.get(`${this.baseUrl}/image-repositories/server/sync`, {})
      .subscribe(response => {
        this.toastService.success('Sincronización completada');
        this.load()
      }, error => {
        console.error('Error al sincronizar', error);
        this.toastService.error('Error al sincronizar');
      });
  }


  toggleAction(image: any, action:string): void {
    switch (action) {
      case 'get-aux':
        this.http.post(`${this.baseUrl}/images/server/${image.uuid}/create-aux-files`, {}).subscribe({
          next: () => {
            this.toastService.success('Petición de creación de archivos auxiliares enviada');
            this.searchImages()
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        });
        break;
      case 'delete':
        this.deleteImage(image);
        break;
      default:
        console.error('Acción no soportada:', action);
        break;
    }
  }

  openImageInfoDialog() {
    this.loadAlert().subscribe(
      response => {
        this.alertMessage = response.output;

        this.dialog.open(ServerInfoDialogComponent, {
          width: '800px',
          data: {
            message: this.alertMessage
          }
        });
      },
      error => {
        console.error('Error al cargar la información del alert', error);
      }
    );
  }
}
